const base = 'http://api.worldbank.org/v2/countries/';
const query = '/indicators/SP.POP.TOTL?format=json';

const countryData = 'https://w3.cs.jmu.edu/kirkpams/343/labs/lab10/countries.json';

/**
 * Asynchronous function for getting the list of countries to add to the
 * #country-list datalist element. Use fetch to retrieve the countryData
 * JSON data set and then parse it. Use the await keyword here to get the
 * actual JSON data as an array. For each element of the array, create a
 * new DOM element and append it to the #country-list. The new element should
 * be the equivalent of the following:
 *
 * <option value="FR">France</option>
 */
async function addValidCountries() {
}

/**
 * Create and return a Promise depending on the results of a call to
 * validateForm. If the form is valid, use resolve() to return an array
 * [country, start, end]. If it is not valid, use reject() to return a
 * string with an error message.
 *
 * @return A new Promise object
 */
function validateQuery() {
}

/**
 * Create and return a new Promise object that fetches a query from the
 * World Bank. Use the base and query global variables along with the
 * criteria in the parameter to construct a URL such as the following:
 *
 * http://api.worldbank.org/v2/countries/USA/indicators/SP.POP.TOTL?format=json&date=1975:2020';
 *
 * This function should call fetch, which returns a Promise for a response.
 * Use the resolve() function to return the response Promise.
 *
 * @param {object} criteria Array containing [ country, start, end ]
 *
 * @return A new Promise object
 */
function fetchCountry(criteria) {
}

/**
 * Create and return a new Promise based on the results of the World Bank
 * query. If json[0].message is defined, then it contains an error message
 * and the query was not successful. In that case, use reject() to return
 * the message value. Otherwise, call redraw() and resolve() the Promise.
 *
 * @param {object} json An array of JSON data with results
 *
 * @return A new Promise object
 */
function processData(json) {
}

/**
 * Initialization function to chain all of the query results together. Add
 * an event listener to the #query-form for submit events. When the form is
 * submitted, use preventDefault and stopPropagation to prevent the reloading
 * of the page. Then, create a chain of Promises (linked with .then() calls)
 * to validate the query form, issue the fetch request, parse the resulting
 * JSON data, then use processData to redraw the results table. Use a .catch()
 * at the end that calls displayError in case any of the Promise objects are
 * rejected.
 *
 * In addition, make a call to addValidCountries() (either before or after
 * adding the event listener) to populate the datalist element with valid
 * country codes.
 */
(function() {
})();


/* ******************************************* *
 * Helper functions below. Do not modify these *
 * ******************************************* */

/**
 * Validates the form fields used to generate queries. Confirms that
 * the start year is less than the end year and both years are within
 * the valid range.
 *
 * @return [valid, country, startValue, endValue] valid is a Boolean
 * that indicates if the form is valid. If so, country is the text of
 * the country code, and startValue and endValue are the years for the
 * start and end date.
 */
function validateForm() {
  let valid = true;

	let form = document.querySelector('#query-form');
	if (!form.classList.contains('was-validated')) {
		form.classList.add('was-validated');
	}
	let start = form['start-date'];
	start.setCustomValidity('');
	let end = form['end-date'];
	end.setCustomValidity('');

  let startValue = 1960;
  let endValue = 2020;

  if (start.value !== '') {
		startValue = parseInt(start.value);
  }
  if (end.value !== '') {
    endValue = parseInt(end.value);
  }

	if (startValue < 1960 || startValue > 2020 || isNaN(startValue)) {
		let msg = document.querySelector('#start-date ~ .invalid-feedback');
		msg.textContent = 'Start date must be a year between 1960 and 2020';
		start.setCustomValidity('bad');
    valid = false;
	}
	if (endValue < 1960 || endValue > 2020 || isNaN(endValue)) {
		let msg = document.querySelector('#end-date ~ .invalid-feedback');
		msg.textContent = 'End date must be a year between 1960 and 2020';
		end.setCustomValidity('bad');
    valid = false;
	}
  if (startValue >= endValue) {
		let msg = document.querySelector('#end-date ~ .invalid-feedback');
		msg.textContent = 'End date must be after start date';
		end.setCustomValidity('bad');
    valid = false;
  }

  let country = form['country-select'].value;

  if (country == '') {
    valid = false;
  }

  return [valid, country, startValue, endValue];
}

/**
 * Takes a JSON object with the results of the World Bank query
 * and redraws the table.
 *
 * @param {object} json The JSON data from the World Bank.
 */
function redraw(json) {
  const list = document.querySelector('#list-data');
  for (let child of Array.from(list.children)) {
    child.remove();
  }
  const title = document.querySelector('#country-header');
  title.textContent = json[1][0].countryiso3code;
  const metadata = document.querySelector('#metadata');
  metadata.textContent = 'Number of rows: ' + json[1].length +
    '; last updated: ' + json[0].lastupdated;

  json[1].forEach(it => list.append(buildEntry(it)));

	setTimeout(() => {
		let form = document.querySelector('#query-form');
		form.classList.remove('was-validated');
		form['start-date'].value = '';
		form['end-date'].value = '';
    form['country-select'].value = '';
	}, 1000);
}

/**
 * Used to display an error message in an alert at the top of the page.
 *
 * @param {string} text The text to show in the alert.
 */
function displayError(text) {
  let container = document.querySelector('#query');
  let msg = document.createElement('p');
  msg.classList.add('alert', 'alert-danger', 'alert-dismissible', 'fade', 'show');
  msg.textContent = text;
  let button = document.createElement('button');
  button.classList.add('btn-close');
  button.setAttribute('data-bs-dismiss', 'alert');
  button.setAttribute('aria-label', 'Close');
  msg.append(button);
  container.insertBefore(msg, container.children[0]);
}

/**
 * Creates an entry for adding to the redrawn list.
 *
 * @param {object} entry The data object to add to the list.
 */
function buildEntry(entry) {
	const entryItem = document.createElement('li');
	entryItem.classList.add('list-group-item', 'py-0');
	const entryRow = document.createElement('div');
	entryRow.classList.add('row');
	entryItem.append(entryRow);
	const entryYear = document.createElement('div');
	entryRow.append(entryYear);
	entryYear.classList.add('col-4', 'border-right', 'text-center');
	entryYear.textContent = entry.date;
	const entryData = document.createElement('div');
	entryRow.append(entryData);
	entryData.classList.add('col', 'text-center');
	if (entry.population !== null) {
		entryData.textContent = formatNumberCommas(entry.value);
	} else {
		entryData.textContent = 'No data';
	}
  return entryItem;
}

/**
 * I can't believe this is necessary, but JavaScript provides no native way to
 * format a number with commas (i.e., 10,324 instead of 10324).
 */
function formatNumberCommas(num) {
  return num.toString().replace(/(\d)(?=(\d{3})+(?!\d))/g, '$1,');
}

